/*
    SDP Editor
    Copyright 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsj
    martin@martin.st
*/

#include "channelchanger.h"
#include "servicesearcher.h"
#include <flogger.h>
#include <aknnotewrappers.h>
#include <textresolver.h>
#include <sdpedit.rsg>

CChannelChanger* CChannelChanger::NewL() {
	CChannelChanger* self = NewLC();
	CleanupStack::Pop(self);
	return self;
}

CChannelChanger* CChannelChanger::NewLC() {
	CChannelChanger* self = new(ELeave) CChannelChanger();
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
}

void CChannelChanger::ConstructL() {
	CActiveScheduler::Add(this);
	iServiceSearcher = new(ELeave) CServiceSearcher();

	iObexFTP = TUUID(0x1106);
	iPCSuite = TUUID(0x00005005, 0x00001000, 0x80000002, 0xee000001);
}

CChannelChanger::CChannelChanger() : CActive(EPriorityStandard) {
	iState = ENotStarted;
	// CSdpAgent doesn't seem to be able to connect to the local
	// SDP server, neither using 00:00:00:ff:ff:ff nor using
	// the actual local bluetooth device address
#ifdef EKA2
	iAddr = TBTDevAddr(TInt64(0x000000FFFFFFL));
#else
	iAddr = TBTDevAddr(TInt64(0x0000, 0x00FFFFFF));
#endif
}

CChannelChanger::~CChannelChanger() {
	Cancel();
	delete iServiceSearcher;
}

void CChannelChanger::UpdateChannelL() {
	if (iState != ENotStarted) {
		HBufC* str = CEikonEnv::Static()->AllocReadResourceAsDes16LC(R_SDPEDIT_IN_PROGRESS);
		ShowWarningL(*str);
		CleanupStack::PopAndDestroy(str);
		return;
	}

	iState = EFindingChannel;
	TRAPD(err, iServiceSearcher->FindServiceChannelL(iPCSuite, iAddr, iStatus));
	if (err != KErrNone) {
		RFileLogger::WriteFormat(_L("debug"), _L("sdpedit.txt"), EFileLoggingModeAppend, _L("FindServiceChannelL: %d"), err);
		ShowWarningL(err);
		iState = ENotStarted;
	} else {
		SetActive();
	}
}

void CChannelChanger::ShowInfoL(const TDesC& aStr) {
	CAknInformationNote* note = new(ELeave) CAknInformationNote(ETrue);
	note->ExecuteLD(aStr);
}

void CChannelChanger::ShowWarningL(const TDesC& aStr) {
	CAknWarningNote* note = new(ELeave) CAknWarningNote(ETrue);
	note->ExecuteLD(aStr);
}

void CChannelChanger::ShowWarningL(TInt aError) {
	CTextResolver* resolver = CTextResolver::NewLC();
#ifdef EKA2
	TPtrC err = resolver->ResolveErrorString(aError);
#else
	TPtrC err = resolver->ResolveError(aError);
#endif
	RFileLogger::WriteFormat(_L("debug"), _L("sdpedit.txt"), EFileLoggingModeAppend, _L("warning: %d"), aError);
	ShowWarningL(err);
	CleanupStack::PopAndDestroy(resolver);
}

void CChannelChanger::RunL() {
	switch (iState) {
	case EFindingChannel: {
		if (iStatus == KErrNotFound) {
			HBufC* str = CEikonEnv::Static()->AllocReadResourceAsDes16LC(R_SDPEDIT_PCSUITE_NOT_FOUND);
			ShowWarningL(*str);
			CleanupStack::PopAndDestroy(str);
			iState = ENotStarted;
			return;
		} else if (iStatus != KErrNone) {
			ShowWarningL(iStatus.Int());
			iState = ENotStarted;
			return;
		}
		iChannel = iServiceSearcher->GetChannel();
		iState = EFindingHandle;
		TRAPD(err, iServiceSearcher->FindServiceHandleL(iObexFTP, iAddr, iStatus));
		if (err != KErrNone) {
			RFileLogger::WriteFormat(_L("debug"), _L("sdpedit.txt"), EFileLoggingModeAppend, _L("FindServiceHandleL: %d"), err);
			ShowWarningL(err);
			iState = ENotStarted;
		} else {
			SetActive();
		}
	}
		break;
	case EFindingHandle:
		if (iStatus == KErrNotFound) {
			HBufC* str = CEikonEnv::Static()->AllocReadResourceAsDes16LC(R_SDPEDIT_OBEXFTP_NOT_FOUND);
			ShowWarningL(*str);
			CleanupStack::PopAndDestroy(str);
			iState = ENotStarted;
			return;
		} else if (iStatus != KErrNone) {
			ShowWarningL(iStatus.Int());
			iState = ENotStarted;
			return;
		}
		iHandle = iServiceSearcher->GetHandle();
		UpdateRecordAndReportL(iHandle, iChannel);
		iState = ENotStarted;
		break;
	default:
		break;
	}
}

void CChannelChanger::DoCancel() {
	switch (iState) {
	case EFindingChannel:
	case EFindingHandle:
		iServiceSearcher->CancelFind();
		break;
	case ENotStarted:
		break;
	default:
		TRequestStatus *status = &iStatus;
		User::RequestComplete(status, KErrNone);
	}
}

void CChannelChanger::UpdateRecordAndReportL(TInt aHandle, TInt aChannel) {
	TRAPD(err, UpdateRecordL(aHandle, aChannel));
	if (err != KErrNone) {
		RFileLogger::WriteFormat(_L("debug"), _L("sdpedit.txt"), EFileLoggingModeAppend, _L("UpdateRecordL: %d"), err);
		ShowWarningL(err);
	} else {
		TBuf<200> buf;
		HBufC* str = CEikonEnv::Static()->AllocReadResourceAsDes16LC(R_SDPEDIT_SUCCESS_FORMAT);
		buf.Format(*str, aHandle, aHandle, aChannel);
		CleanupStack::PopAndDestroy(str);
		ShowInfoL(buf);
	}
}

void CChannelChanger::UpdateRecordL(TInt aHandle, TInt aChannel) {
	TBuf8<1> channel;
	channel.Append(aChannel);

	RSdp sdpSession;
	RSdpDatabase sdpDatabase;
	User::LeaveIfError(sdpSession.Connect());
	CleanupClosePushL(sdpSession);
	User::LeaveIfError(sdpDatabase.Open(sdpSession));
	CleanupClosePushL(sdpDatabase);

	CSdpAttrValueDES* descriptor = CSdpAttrValueDES::NewDESL(NULL);
	CleanupStack::PushL(descriptor);
	descriptor
		->StartListL()
		->BuildDESL()
		->StartListL()
			->BuildUUIDL(KL2CAP)
		->EndListL()
		->BuildDESL()
		->StartListL()
			->BuildUUIDL(KRFCOMM)
			->BuildUintL(channel)
		->EndListL()
		->BuildDESL()
		->StartListL()
			->BuildUUIDL(KOBEX)
		->EndListL()
	->EndListL();

	sdpDatabase.UpdateAttributeL(aHandle, KSdpAttrIdProtocolDescriptorList, *descriptor);

	CleanupStack::PopAndDestroy(descriptor);
	CleanupStack::PopAndDestroy(&sdpDatabase);
	CleanupStack::PopAndDestroy(&sdpSession);
}

